#Source: https://github.com/asciidoctor/asciidoctor-extensions-lab/blob/master/lib/emoji-inline-macro/extension.rb
#
require 'asciidoctor/extensions' unless RUBY_ENGINE == 'opal'
require 'erb'
require 'tomlrb'

include Asciidoctor

class P2EDocinfoProcessor < Extensions::DocinfoProcessor
  use_dsl

  def process doc
    srcdir = ::File.join(::File.dirname __FILE__)
    outdir = (doc.attr? 'outdir') ? (doc.attr 'outdir') : (doc.attr 'docdir')
    dstdir = "#{outdir}/p2e"
    FileUtils.mkdir_p(dstdir)
    FileUtils.cp_r("#{srcdir}/assets/.", dstdir)
    doc.read_asset("#{srcdir}/docinfo.html")
  end
end

class P2EStatblockProcessor < Extensions::BlockProcessor
  use_dsl

  named :stat
  on_context :open, :example
  parse_content_as :raw

  def process parent, reader, attrs
    # parse TOML
    lines = reader.lines
    data = Tomlrb.parse(lines * "\n")

    # get/set attributes
    if attrs.key?('type')
      type = attrs['type']
    else
      type = 'creature'
    end
    if attrs.key?('role')
      data['role'] = attrs['role']
    end

    # create HTML from template
    extdir = ::File.join(::File.dirname __FILE__)
    templates = {
      'creature' => "#{extdir}/creatureblock.html.erb",
      'item' => "#{extdir}/itemblock.html.erb",
      'hazard' => "#{extdir}/hazardblock.html.erb"
    }
    template = File.read(templates[type])
    html = ERB.new(template).result_with_hash(data)
    create_pass_block parent, html, attrs, subs: nil
  end
end
